const os = require("os");
const fs = require('fs-extra');
const path = require('path');
const crypto = require('crypto');
const https = require('https');
const AdmZip = require('adm-zip');

const isOSWindows = process.platform === "win32";
const baseDirName = "Jasperactive-Adobe";
const homeDir = os.homedir().replace(/\\/g, '/');
const publicDir = isOSWindows ? process.env['PUBLIC'].replace(/\\/g, '/') : "/Users/Shared";

class FileManager {
	constructor({ appDirName, startersDirName, resourcesDirName, myProjectsDirName, langCode }) {

		this.dirInfo = {
			starterDir: "StarterFiles",
			resourceDir: "ResourceFiles"
		};

		const publicDocuments = isOSWindows ? "documents/" : "";

		this.publicDirInfo = {
			baseDir: `${publicDir}/${publicDocuments}${baseDirName}`,
			appDir: `${publicDir}/${publicDocuments}${baseDirName}/${appDirName}`,
			startersDir: `${publicDir}/${publicDocuments}${baseDirName}/${appDirName}/${this.dirInfo.starterDir}`,
			startersLangDir: `${publicDir}/${publicDocuments}${baseDirName}/${appDirName}/${this.dirInfo.starterDir}/${langCode}`,
			resourcesDir: `${publicDir}/${publicDocuments}${baseDirName}/${appDirName}/${this.dirInfo.resourceDir}`
		};

		this.homeDirInfo = {
			baseDir: `${homeDir}/documents/${baseDirName}`,
			appDir: `${homeDir}/documents/${baseDirName}/${appDirName}`,
			startersDir: `${homeDir}/documents/${baseDirName}/${appDirName}/${startersDirName}`,
			resourcesDir: `${homeDir}/documents/${baseDirName}/${appDirName}/${resourcesDirName}`,
			myProjectsDir: `${homeDir}/documents/${baseDirName}/${appDirName}/${myProjectsDirName}`
		};
	}

	checkDirsAndDeleteHomeFiles() {
		this.checkAndCreateDirectory(this.publicDirInfo.baseDir, 0o777);
		this.checkAndCreateDirectory(this.publicDirInfo.appDir, 0o777);
		this.checkAndCreateDirectory(this.publicDirInfo.startersDir, 0o777);
		this.checkAndCreateDirectory(this.publicDirInfo.startersLangDir, 0o777);
		this.checkAndCreateDirectory(this.publicDirInfo.resourcesDir, 0o777);

		this.deleteDirectory(this.homeDirInfo.appDir);
		this.checkAndCreateDirectory(this.homeDirInfo.baseDir);
		this.checkAndCreateDirectory(this.homeDirInfo.appDir);
		this.checkAndCreateDirectory(this.homeDirInfo.startersDir);
		this.checkAndCreateDirectory(this.homeDirInfo.resourcesDir);
		this.checkAndCreateDirectory(this.homeDirInfo.myProjectsDir);

		return this.deleteFiles(this.homeDirInfo.appDir);
	}

	checkAndCreateDirectory(dirPath, mode) {
		if (!fs.existsSync(dirPath)) {
			fs.mkdirSync(dirPath);
			if (mode) {
				fs.chmodSync(dirPath, mode);
			}
		}
	}

	deleteFiles(dir) {
		try {
			if (!fs.existsSync(dir)) {
				return { ok: true };
			}

			const files = this.getDirFiles(dir);

			files.forEach(file => {
				if (file.indexOf(baseDirName) > -1) {
					fs.unlinkSync(file);
				}
			});

		} catch (error) {
			return { ok: false, errorMessage: error.message };
		}
		return { ok: true };
	}

	deleteDirectory(dir) {
		try {
			fs.emptyDirSync(dir);
		} catch (error) { console.error(error); }
		return { ok: true };
	}

	getDirFiles(dirPath, arrayOfFiles) {
		const files = fs.readdirSync(dirPath)

		arrayOfFiles = arrayOfFiles || [];

		for (let i = 0; i < files.length; i++) {
			const file = files[i];

			if (fs.statSync(dirPath + "/" + file).isDirectory()) {
				arrayOfFiles = this.getDirFiles(dirPath + "/" + file, arrayOfFiles)
			} else {
				arrayOfFiles.push(path.join(dirPath, "/", file))
			}
		}

		return arrayOfFiles
	}

	async processFiles(publicDir, homeDir, items) {
		const fileInfo = [];

		for (let i = 0; i < items.length; i++) {
			const item = items[i];
			const filePublicPath = `${publicDir}/${item.name}`;
			const checksum = await this.getFileChecksum(filePublicPath);
			const isFileNew = checksum === null;

			if (item.checksum !== checksum) {
				const url = new URL(item.url);
				const zipfileName = path.basename(url.pathname);
				const zipFilePublicPath = `${publicDir}/${zipfileName}`;

				await this.downloadFile(item.url, zipFilePublicPath);
				this.extractFile(zipFilePublicPath, item.name, zipFilePublicPath.replace(`/${zipfileName}`, ''));
				fs.unlinkSync(zipFilePublicPath);
			}

			const fileLocalPath = `${homeDir}/${item.name}`;

			if (isFileNew) {
				fs.chmodSync(filePublicPath, 0o777);
			}

			fs.copyFileSync(filePublicPath, fileLocalPath);

			fileInfo.push({
				localPath: fileLocalPath,
				autoOpen: item.autoOpen,
				isFileNew
			});
		}

		return fileInfo;
	}

	getFileChecksum(filePath) {
		return new Promise((resolve, reject) => {
			if (fs.existsSync(filePath)) {
				const hash = crypto.createHash('md5');
				const input = fs.createReadStream(filePath);

				input.on('error', reject);

				input.on('data', chunk => {
					hash.update(chunk);
				});

				input.on('close', function () {
					resolve(hash.digest('hex'));
				});
			}
			else {
				resolve(null);
			}
		});
	}

	downloadFile(url, localPath) {
		return new Promise((resolve, reject) => {
			try {
				if (!url.toLowerCase().startsWith("https://addin-resources.jasperactive.com/")) {
					throw Error(`The address '${url}' is not allowed`);
				}

				https.get(url, response => {
					const file = fs.createWriteStream(localPath);
					response.pipe(file);
					file.on('finish', () => {
						file.close();
						resolve();
					})
				})
			} catch (error) {
				reject(error);
			}
		})
	}

	extractFile(zipFilePath, entryName, targetPath) {
		var zipFile = new AdmZip(zipFilePath);
		zipFile.extractEntryTo(entryName, targetPath, true, true);
	}
}

module.exports = {
	baseDirName,
	homeDir,
	publicDir,
	isOSWindows,
	FileManager
};
